`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: Robert Kent
// 
// Create Date: 08/22/2023 2:20 PM
// Design Name: 
// Module Name: dac_fast
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: Code for DAC8551

// Power-Down modes:

//PD1 PD0 OPERATING MODE
//0 0 Normal operation
//Power-down modes
//0 1 Output typically 1 kΩ to GND
//1 0 Output typically 100 kΩ to GND
//1 1 High-Z

// Future Optimizations:

// there is more time to be saved with t8 and t9 from datasheet

// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module dac_fast(
    input wire CLK, // 20MHz clock with 41% duty cycle
    input wire RESET, // active low
    input wire [15:0] DAC_DATA, // 16-bit data from user to convert to analog signal
    output reg SYNC, 
    output wire SCLK,
    output reg DIN,
    output wire [7:0] CNT2, // for syncing DAC_DATA input
	 output reg DAC_FLAG // High when DIN = First or Last data bit
    );

//=======================================================
//  REG/WIRE declarations
//======================================================= 

// Power-Down Mode
wire PD1 = 0/* synthesis keep */;
wire PD0 = 0/* synthesis keep */;
reg [15:0] DAC_DATA_LOCKED = 0/* synthesis noprune */;
//49151 then 16384 (75% then 25% of max), ~1011111111111111 = 0100000000000000

// State Counter
reg [4:0] CNT/* synthesis noprune */;

//=======================================================
//  Structural coding
//=======================================================

// Creating copy of CLK as output
assign SCLK = CLK;

// Creating copy of CNT as output
assign CNT2 = CNT;

// Testing outputs

// Square wave
//reg [4:0] square_counter = 0;
//localparam square_high_cycles = 20;
//always @(posedge CLK)
//begin
//	if (CNT == 0)
//	begin
//		if (square_counter < square_high_cycles - 1) // WIP -1
//			square_counter = square_counter + 1;
//		else
//		begin
//			square_counter = 0;
//			DAC_DATA_LOCKED = ~DAC_DATA_LOCKED;
//		end
//	end
//end

// Alternate every other
//reg toggle = 0/* synthesis noprune */;
//always @(posedge CLK) begin
//    if (CNT == 0) begin
//        // Toggle the value of DAC_DATA_LOCKED between USER_SPECIFIED_VALUE_1 and USER_SPECIFIED_VALUE_2
//        if (toggle)
//            //DAC_DATA_LOCKED <= 39321;//60%
//				DAC_DATA_LOCKED <= 36044; //55%
//        else
//            //DAC_DATA_LOCKED <= 26214;//40%
//				DAC_DATA_LOCKED <= 29491;//45%
//        
//        // Toggle the 'toggle' variable to switch between values
//        toggle <= ~toggle;
//    end
//end

// Main Code
always @(negedge RESET or posedge CLK)
begin
	if (!RESET)
	begin
		SYNC = 1; // High for CNT == 25 and CNT == 0
		DIN = 0;
		CNT = 0;
	end

	else 
	begin
	case (CNT)
	5'd0:
		// no DIN bits are sent during CNT == 0
		// Unused (bit 23)
	   begin
	       SYNC = 0;
	       DIN = 0;
	       CNT = CNT + 1;
	   end
	5'd1:
		// Unused (bit 22)
	   begin
	       DIN = 0;
	       CNT = CNT + 1;
	   end
	5'd2:
		// Unused (bit 21)
	   begin
	       DIN = 0;
	       CNT = CNT + 1;
	   end   
	5'd3:
		// Unused (bit 20)
	   begin
	       DIN = 0;
	       CNT = CNT + 1;
	   end   
	5'd4:
		// Unused (bit 19)
	   begin
	       DIN = 0;
	       CNT = CNT + 1;
	   end   
	5'd5:
		// Unused (bit 18)
	   begin
	       DIN = 0;
	       CNT = CNT + 1;
	   end   
	5'd6:
		// Power mode PD1 (bit 17)
	   begin
	       DIN = PD1;
	       CNT = CNT + 1;
			 // Update Data Ready Signal? Can we make this sync with RSPN_VALID?
	   end   
	5'd7:
		// Power mode PD0 (bit 16)
	   begin
	       DIN = PD0;
	       CNT = CNT + 1;
			 // Update Data bits with DAC_DATA
			 //DAC_DATA_LOCKED = DAC_DATA;
			 //Testing maximum switching rate
			 //DAC_DATA_LOCKED = ~DAC_DATA_LOCKED;
	   end   
	5'd8:
		// Data bit (bit 15)
	   begin
			 DAC_DATA_LOCKED = DAC_DATA; // WIP: moved one state later
			 DAC_FLAG = 1;
	       DIN = DAC_DATA_LOCKED[15];
	       CNT = CNT + 1;
	   end   
	5'd9:
		// Data bit (bit 14)
	   begin
			 DAC_FLAG = 0;
	       DIN = DAC_DATA_LOCKED[14];
	       CNT = CNT + 1;
	   end   
	5'd10:
		// Data bit (bit 13)
	   begin
	       DIN = DAC_DATA_LOCKED[13];
	       CNT = CNT + 1;
	   end   
	5'd11:
		// Data bit (bit 12)
	   begin
	       DIN = DAC_DATA_LOCKED[12];
	       CNT = CNT + 1;
	   end   
	5'd12:
		// Data bit (bit 11)
	   begin
	       DIN = DAC_DATA_LOCKED[11];
	       CNT = CNT + 1;
	   end   
	5'd13:
		// Data bit (bit 10)
	   begin
	       DIN = DAC_DATA_LOCKED[10];
	       CNT = CNT + 1;
	   end   
	5'd14:
		// Data bit (bit 9)
	   begin
	       DIN = DAC_DATA_LOCKED[9];
	       CNT = CNT + 1;
	   end   
	5'd15:
		// Data bit (bit 8)
	   begin
	       DIN = DAC_DATA_LOCKED[8];
	       CNT = CNT + 1;
	   end   
	5'd16:
		// Data bit (bit 7)
	   begin
	       DIN = DAC_DATA_LOCKED[7];
	       CNT = CNT + 1;
	   end   
	5'd17:
		// Data bit (bit 6)
	   begin
	       DIN = DAC_DATA_LOCKED[6];
	       CNT = CNT + 1;
	   end   
	5'd18:
		// Data bit (bit 5)
	   begin
	       DIN = DAC_DATA_LOCKED[5];
	       CNT = CNT + 1;
	   end   
	5'd19:
		// Data bit (bit 4)
	   begin
	       DIN = DAC_DATA_LOCKED[4];
	       CNT = CNT + 1;
	   end   
	5'd20:
		// Data bit (bit 3)
	   begin
	       DIN = DAC_DATA_LOCKED[3];
	       CNT = CNT + 1;
	   end   
	5'd21:
		// Data bit (bit 2)
	   begin
	       DIN = DAC_DATA_LOCKED[2];
	       CNT = CNT + 1;
	   end   
	5'd22:
		// Data bit (bit 1)
	   begin
	       DIN = DAC_DATA_LOCKED[1];
	       CNT = CNT + 1;
	   end   
	5'd23:
		// Data bit (bit 0)
	   begin
			 DAC_FLAG = 1;
	       DIN = DAC_DATA_LOCKED[0];
	       CNT = CNT + 1;
	   end   
	5'd24:
		// SYNC high for two cycles (>100ns)
		// edit: d25 is 2 clock cycles, d26 is 3
	   begin
			 DAC_FLAG = 0;
	       SYNC = 1;
	       CNT = CNT + 1;
	   end   
	5'd25:
	   begin
	       CNT = 0;
	   end 
    endcase
    end
end
 
endmodule